 #include <xc.h>

#include "pic16f886.h"
#define _XTAL_FREQ 8000000


// CONFIG1
#pragma config FOSC = INTRC_NOCLKOUT// Oscillator Selection bits (INTOSCIO oscillator: I/O function on RA6/OSC2/CLKOUT pin, I/O function on RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled and can be enabled by SWDTEN bit of the WDTCON register)
#pragma config PWRTE = OFF  	// Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = OFF  	// RE3/MCLR pin function select bit (RE3/MCLR pin function is digital input, MCLR internally tied to VDD)
#pragma config CP = OFF  	   // Code Protection bit (Program memory code protection is disabled)
#pragma config CPD = OFF 	   // Data Code Protection bit (Data memory code protection is disabled)
#pragma config BOREN = OFF  	// Brown Out Reset Selection bits (BOR disabled)
#pragma config IESO = OFF       // Internal External Switchover bit (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF  	// Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is disabled)
#pragma config LVP = OFF 	   // Low Voltage Programming Enable bit (RB3 pin has digital I/O, HV on MCLR must be used for programming)

// CONFIG2
#pragma config BOR4V = BOR40V   // Brown-out Reset Selection bit (Brown-out Reset set to 4.0V)
#pragma config WRT = OFF 	   // Flash Program Memory Self Write Enable bits (Write protection off)

#define TX PORTAbits.RA7
#define sw_reset PORTAbits.RA4


//POSICIONES DE BIT
#define POS_PL1   0b00000001
#define POS_PL2   0b00000010
#define POS_BT1   0b00000100
#define POS_BT2   0b00001000
#define POS_BT3   0b00010000
#define POS_BT4   0b00100000

// BITS DE SWITCHES
#define PORT_PL1   PORTBbits.RB4
#define PORT_PL2   PORTBbits.RB5
#define PORT_BT1   PORTBbits.RB6
#define PORT_BT2   PORTCbits.RC3
#define PORT_BT3   PORTBbits.RB2
#define PORT_BT4   PORTCbits.RC2


char data_time, i, inter_time, est=0;
int cont;
 
unsigned int pwm, pot, recibido2, direccion;
unsigned int resultado;

void send_data (void);
void init(void);
void icio (void);

 
unsigned char info[5];
//info[0]=sync
//info[1]=adelante/atras PWM
//info[2]=izq o derecha
//info[3]=switches
//info[4]=checksum

void main (){
    init ();
	icio ();
    unsigned char checksum;  //Dato de 1 byte

   
    
    while (1){
   	
    info[0]=0b01010101;
    
 	checksum=info[1]^info[2];	 
 	checksum=checksum^info[3];
    info[4]=checksum;
    

    send_data();
	  
	 
  	  ADCON0bits.CHS = 0b0010;//AN2   
   	 __delay_ms(5); //Tiempo muerto entre envio de data, y de paso se carga el ADC para la proxima conversion
   	 ADCON0bits.GO_nDONE=1;
  	  while(ADCON0bits.GO_nDONE==1){}
   	 pot = ADRESH << 8; //Corro los 2 bits para que adresL no los pise
   	 pot&= 0x0300; //Se encarga de que no se llene de basura el dato pot al correr 8 bits
   	 pot += ADRESL; //Se agregan los bits de adresl a pot
   	 pwm = pot >> 2; //Divido entre 4 para que el maximo sea 255 y no 1024 partes
   	 info[1]=pwm;
  	 
     //BOTONES 
     if(RA4==1)est=0;	 //RESET, ES NC
  	 if(est==0 && RA4==0){ 
         est=1; 
         asm("goto 0"); //Sentencia en asembler, que reinicia el programa al bit 0 de memoria
     }

     //PALANCA 1 
     if (PORT_PL1 == 1){
         info[3] |= POS_PL1; //PONE EN 1 EL BIT DE LA PALANCA
     }
     else{
         info[3] &= ~POS_PL1;//PONE EN 0 EL BIT DE LA PALANCA
     }
     
     //PALANCA 2   
     if (PORT_PL2 == 1){
         info[3] |= POS_PL2;
     }
     else{
         info[3] &= ~POS_PL2;//PONE EN 0 EL BIT DE LA PALANCA
     }
     
    
     if (PORT_BT1 == 1){
         info[3] |= POS_BT1;
     }
     else{
         info[3] &= ~POS_BT1;//PONE EN 0 EL BIT DE LA PALANCA
     }
     
     //BOTON 2 
     if (PORT_BT2 == 1){
         info[3] |= POS_BT2;
     }
     else{
         info[3] &= ~POS_BT2;//PONE EN 0 EL BIT DE LA PALANCA
     }
     
     
     //BOTON 3
     if (PORT_BT3 == 1){
         info[3] |= POS_BT3;
     }
     else{
         info[3] &= ~POS_BT3;//PONE EN 0 EL BIT DE LA PALANCA
     }

     
     //BOTON 4
     if (PORT_BT4 == 1){
         info[3] |= POS_BT4;
     }
     else{
         info[3] &= ~POS_BT4;//PONE EN 0 EL BIT DE LA PALANCA
     }

     
  	  ADCON0bits.CHS = 0b0011; //AN3
   	 __delay_ms(5);
   	 ADCON0bits.GO_nDONE=1;
  	  while(ADCON0bits.GO_nDONE==1){}
  	   //Lo pongo ac para que haya un tiempo entre convesiones, ya que los componentesw analogicos (Capacitores, etc) se tienen que reestablecer
   	 recibido2 = ADRESH << 8; //Corro los 2 bits para que adresL no los pise
   	 recibido2&= 0x0300; //Se encarga de que no se llene de basura el dato pot al correr 8 bits
   	 recibido2 += ADRESL; //Se agregan los bits de adresl a pot
   	 direccion = recibido2 >> 2; //Divido entre 4 para que el maximo sea 255 y no 1024 partes
   	 info[2]=direccion;

     //6mS tarda en enviar un byte, y tengo uno de sincronizacin y otro de data, yo quiero un refresco de 20 veces por segundo, o sea cada 50mS - 2*6mS = 38mS, o sea, cada 38ms, 38mS/512Us=74Veces
    }
    
}

void init (){
    /*I/O*/
    TRISA = 0b00111101; //RA7 es TX y RA0, RA5, RA4 Input
    TRISB = 0b11111111;
    TRISC = 0b00001100; //pines en entrada son los switches
	ANSELH=0; //Todas las dems entrada digitales
    ANSEL=12; //Configura todos los pines de entrada como digiales menos AN2 y AN3 MUY IMPORTANTE, paa cambiar esto se usa el registro ansel y anselh
	ADCON0 = 0b10001001; //-10- fosc/32 para el tad (tiempo de adquisicion)/ -0010- Selecciono canal AN2/-0- este bit configura proceso de lectura/ -1- este bit activa el adc
	ADCON1 = 0b10000000; //Justifico los bits a la derecha, para que los bits mas significativos sean los 2 de adresh
    
    /*FREQ DE OPERACIN*/
    OSCCONbits.IRCF=7; //8 MHZ
    
    /*Timer*/
    OPTION_REGbits.T0CS=0; //Activo la opcion de usar el oscilador interno
    //OPTION_REGbits.PSA=0; //Desactivacin del preescaler
    //OPTION_REGbits.PS=0b000 ; //El preesclaer divide la frecuencia entre 2, dando 256uS hasta el desborde, para que en el 16f628a no haga falta usarlo
    
    /*Interrupciones*/
    GIE=1; //interrupciones
    T0IF=0; //por las dudas bajo la bandera del timer
    T0IE=0; //Interrupciones de tmr1
    

}

void icio (void){
	TX=1;
	__delay_ms(60);
}
void send_data (void){/*aca tengo que mandar los bits en tiempo me falta agregar los bits de protocolo*/
    char a;
    
    for(a=0; a<5; a++){
        TX=1;
        for (i=0; i<8; i++){
            TX=0;
            if ((info[a]&1)==1){
                __delay_us(508);    
                TX=1;
                __delay_us(254);
            }	 
            else{    
     	   __delay_us(254);
     	   TX=1;
     	   __delay_us(508);
           }  
            info[a]=info[a]>>1;
        }
    }   
}






